@@ -25,13 +25,16 @@ use warnings;
 use autodie;
 
 use List::MoreUtils qw(any);
-use Moo;
+use Path::Tiny;
 
 use Lintian::Data;
 use Lintian::Maintainer qw(check_maintainer);
 use Lintian::Util qw(get_file_checksum);
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 my $KNOWN_DISTS = Lintian::Data->new('changes-file/known-dists');
 
@@ -188,11 +191,12 @@ sub changes {
     # Changed-By is optional in Policy, but if set, must be
     # syntactically correct.  It's also used by dak.
     if ($info->field('changed-by')) {
-        check_maintainer($info->field('changed-by'), 'changed-by');
+        my @tags = check_maintainer($info->field('changed-by'), 'changed-by');
+        $self->tag(@{$_}) for @tags;
     }
 
     my $files = $info->files;
-    my $path = readlink($info->lab_data_path('changes'));
+    my $path = readlink(path($info->groupdir)->child('changes')->stringify);
     my %num_checksums;
     $path =~ s#/[^/]+$##;
     foreach my $file (keys %$files) {
