@@ -30,18 +30,15 @@ use warnings;
 use autodie;
 use v5.10;
 
-use Moo;
-
 use File::Basename qw(basename);
 use List::MoreUtils qw(any);
+use Path::Tiny;
 
 use Lintian::Data;
 use Lintian::Relation ();
 use Lintian::Util qw(internal_error normalize_pkg_path strip open_gz);
 use Lintian::SlidingWindow;
 
-with('Lintian::Check');
-
 # Half of the size used in the "sliding window" for detecting bad
 # licenses like GFDL with invariant sections.
 # NB: Keep in sync cruft-gfdl-fp-sliding-win/pre_build.
@@ -52,6 +49,11 @@ use constant BLOCKSIZE => 16_384;
 use constant INSANE_LINE_LENGTH => 512;
 use constant SAFE_LINE_LENGTH => 256;
 
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
+
 our $LIBTOOL = Lintian::Relation->new('libtool | dh-autoreconf');
 
 # load data for md5sums based check
@@ -426,7 +428,7 @@ sub source {
     # package if it isn't just tar cruft.
     for my $file (keys %ERRORS) {
         my $tag  = $ERRORS{$file};
-        my $path = $info->lab_data_path($file);
+        my $path = path($info->groupdir)->child($file)->stringify;
         if (-s $path) {
             open(my $fd, '<', $path);
             local $_;
@@ -1768,7 +1770,7 @@ sub _ships_examples {
         return 1
           if any { m{^usr/share/doc/(.+/)?examples/?$} } @files;
     }
-    return;
+    return 0;
 }
 
 1;
