@@ -26,14 +26,16 @@ use autodie;
 
 use List::MoreUtils qw(any);
 use List::Util qw(first none);
-use Moo;
 
 use Lintian::Data ();
 use Lintian::Deb822Parser qw(read_dpkg_control);
 use Lintian::Relation ();
 use Lintian::Util qw(file_is_encoded_in_non_utf8 rstrip strip);
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 # The list of libc packages, used for checking for a hard-coded dependency
 # rather than using ${shlibs:Depends}.
@@ -482,7 +484,7 @@ sub source {
           unless $info->source_field('xs-go-import-path', '');
     }
 
-    my $changes = $group->get_changes_processable;
+    my $changes = $group->changes;
     $self->tag('source-only-upload-to-non-free-without-autobuild')
       if defined($changes)
       and $changes->info->field('architecture', '') eq 'source'
