@@ -25,6 +25,19 @@ use strict;
 use warnings;
 use autodie;
 
+use Encode qw(decode);
+use List::Compare;
+use List::MoreUtils qw(any none);
+use Path::Tiny;
+use Text::Levenshtein qw(distance);
+use XML::Simple qw(:strict);
+
+use Lintian::Data;
+use Lintian::Deb822Parser qw(read_dpkg_control parse_dpkg_control);
+use Lintian::Relation::Version qw(versions_compare);
+use Lintian::Spelling qw(check_spelling);
+use Lintian::Util qw(file_is_encoded_in_non_utf8);
+
 use constant {
     DH_MAKE_TODO_BOILERPLATE_1 =>join(q{ },
         '# Please also look if there are files or directories',
@@ -36,37 +49,23 @@ use constant {
         'Delete these two lines'),
 };
 
-use Encode qw(decode);
-use List::Compare;
-use List::MoreUtils qw(any none);
-use Moo;
-use Path::Tiny;
-use Text::Levenshtein qw(distance);
-use XML::Simple qw(:strict);
-
-use Lintian::Data;
-use Lintian::Deb822Parser qw(read_dpkg_control parse_dpkg_control);
-use Lintian::Relation::Version qw(versions_compare);
-use Lintian::Spelling qw(check_spelling spelling_tag_emitter);
-use Lintian::Util qw(file_is_encoded_in_non_utf8);
-
-with('Lintian::Check');
-
-use constant SPACE => q{ };
-
 use constant {
     WC_TYPE_REGEX => 'REGEX',
     WC_TYPE_FILE => 'FILE',
     WC_TYPE_DECENDANTS => 'DECENDANTS',
 };
 
+use constant SPACE => q{ };
+
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
+
 our $KNOWN_ESSENTIAL = Lintian::Data->new('fields/essential');
 our $KNOWN_COMMON_LICENSES
   =  Lintian::Data->new('copyright-file/common-licenses');
 
-my $SPELLING_ERROR_IN_COPYRIGHT
-  = spelling_tag_emitter('spelling-error-in-copyright');
-
 my $BAD_SHORT_LICENSES = Lintian::Data->new(
     'source-copyright/bad-short-licenses',
     qr/\s*\~\~\s*/,
@@ -87,6 +86,13 @@ my %dep5_renamed_fields        = (
     'name'                 => 'upstream-name',
 );
 
+sub spelling_tag_emitter {
+    my ($self, @orig_args) = @_;
+    return sub {
+        return $self->tag(@orig_args, @_);
+    };
+}
+
 sub source {
     my ($self) = @_;
 
@@ -881,7 +887,7 @@ sub binary {
         return;
     }
 
-    my $dcopy = $info->lab_data_path('copyright');
+    my $dcopy = path($info->groupdir)->child('copyright')->stringify;
     # check that copyright is UTF-8 encoded
     my $line = file_is_encoded_in_non_utf8($dcopy);
     if ($line) {
@@ -1103,8 +1109,11 @@ qr/GNU (?:Lesser|Library) General Public License|(?-i:\bLGPL\b)/i
                |\bpublic(?:\s+|-)domain\b/xi;
     }
 
-    check_spelling($_, $group->info->spelling_exceptions,
-        $SPELLING_ERROR_IN_COPYRIGHT, 0);
+    check_spelling(
+        $_,
+        $group->info->spelling_exceptions,
+        $self->spelling_tag_emitter('spelling-error-in-copyright'), 0
+    );
 
     # Now, check for linking against libssl if the package is covered
     # by the GPL.  (This check was requested by ftp-master.)  First,
@@ -1149,7 +1158,7 @@ sub check_cross_link {
     my ($self, $fpkg) = @_;
 
     my $group = $self->group;
-    my $src = $group->get_source_processable;
+    my $src = $group->source;
     if ($src) {
         # source package is available; check it's list of binary
         return if defined $src->info->binary_package_type($fpkg);
