@@ -18,16 +18,18 @@ package Lintian::debian::rules;
 use strict;
 use warnings;
 use autodie;
-use Carp qw(croak);
 
+use Carp qw(croak);
 use List::MoreUtils qw(any none);
-use Moo;
 
 use Lintian::Data;
 use Lintian::SlidingWindow;
 use Lintian::Util qw(rstrip);
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 our $PYTHON_DEPEND
   = 'python:any | python-dev:any | python-all:any | python-all-dev:any | python2:any | python2-dev:any';
@@ -464,6 +466,7 @@ sub source {
     if (my $line = $overridden{'dh_auto_test'}) {
         my @lines = grep {
                   $_ !~ m{^\t\s*[\:\[]}
+              and $_ !~ m{^\s*$}
               and $_ !~ m{\bdh_auto_test\b}
               and $_
               !~ m{^\t\s*[-@]?(?:(?:/usr)?/bin/)?(?:cp|chmod|echo|ln|mv|mkdir|rm|test|true)}
