@@ -30,7 +30,6 @@ use autodie;
 
 use Capture::Tiny qw(capture);
 use List::MoreUtils qw(any);
-use Moo;
 use POSIX qw(strftime);
 use Try::Tiny;
 
@@ -39,7 +38,10 @@ use Lintian::Data;
 use Lintian::Relation;
 use Lintian::Util qw(internal_error safe_qx strip);
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 # This is a map of all known interpreters.  The key is the interpreter
 # name (the binary invoked on the #! line).  The value is an anonymous
@@ -311,13 +313,13 @@ sub binary {
           if (  $filename =~ m,\.in$,
             and $interpreter =~ m,^(\@|<\<)[A-Z_]+(\@|>\>)$,);
 
-        my $is_absolute = ($interpreter =~ m,^/, or defined $calls_env);
+        my $is_absolute = ($interpreter =~ m,^/, or $calls_env);
 
         # As a special-exception, Policy 10.4 states that Perl scripts must use
         # /usr/bin/perl directly and not via /usr/bin/env, etc.
         $self->script_tag(bad_interpreter_tag_name('/usr/bin/env perl'),
             $filename, '(#!/usr/bin/env perl != /usr/bin/perl)')
-          if defined $calls_env and $interpreter eq 'perl';
+          if $calls_env and $interpreter eq 'perl';
 
         # Skip files that have the #! line, but are not executable and
         # do not have an absolute path and are not in a bin/ directory
@@ -414,7 +416,7 @@ sub binary {
         }
         if ($data) {
             my $expected = $data->[0] . '/' . $base;
-            unless ($interpreter eq $expected or defined $calls_env) {
+            unless ($interpreter eq $expected or $calls_env) {
                 $self->script_tag(bad_interpreter_tag_name($expected),
                     $filename, "(#!$interpreter != $expected)");
             }
@@ -442,7 +444,7 @@ sub binary {
                 # executable).
                 my $interfile = substr $interpreter, 1;
                 $pinter = 1 if $executable{$interfile};
-            } elsif (defined $calls_env) {
+            } elsif ($calls_env) {
                 for my $dir (qw(usr/bin bin)) {
                     if ($executable{"$dir/$interpreter"}) {
                         $pinter = 1;
@@ -585,7 +587,8 @@ sub binary {
           );
     }
 
-    open(my $ctrl_fd, '<', $info->lab_data_path('control-scripts'));
+    # get maintainer scripts
+    my %control = %{$self->info->control_scripts};
 
     # Handle control scripts.  This is an edited version of the code for
     # normal scripts above, because there were just enough differences to
@@ -593,14 +596,10 @@ sub binary {
 
     my (%added_diversions, %removed_diversions, %dh_cmd_substs);
     my $expand_diversions = 0;
-    while (<$ctrl_fd>) {
-        chop;
+    for my $file (keys %control) {
 
-        m/^(\S*) (.*)$/
-          or internal_error("bad line in control-scripts file: $_");
-        my $interpreter = $1;
-        my $file = $2;
         my $path = $info->control_index_resolved_path($file);
+        my $interpreter = $control{$file};
 
         $interpreter =~ m|([^/]*)$|;
         my $base = $1;
@@ -1185,7 +1184,6 @@ m,$LEADIN(?:/usr/bin/)?dpkg\s+--compare-versions\s+.*\b\Q$ver\E(?!\.)\b,
         close($fd);
 
     }
-    close($ctrl_fd);
 
     for my $cmd (qw(rm_conffile mv_conffile symlink_to_dir)) {
         next unless $seen_helper_cmds{$cmd};
