@@ -160,7 +160,7 @@ inputs.
 sub is_check_type {
     my ($self, $type) = @_;
     return 1 if ($self->{'type'}//'ALL') eq 'ALL';
-    return $self->{'type-table'}{$type};
+    return $self->{'type-table'}{$type} // 0;
 }
 
 =item $cs->add_taginfo ($taginfo)
@@ -202,10 +202,7 @@ sub tags {
 =item $cs->run_check ($proc, $group)
 
 Run the check on C<$proc>, which is in the
-L<group|Lintian::Processable::Group> C<$group>.  C<$proc> should be
-a L<lab entry|Lintian::Lab::Entry> and must have the proper
-collections run on it prior to calling this method (See
-L<Lintian::Unpacker>).
+L<group|Lintian::Processable::Group> C<$group>.
 
 The method may error out if loading the check failed or if the check
 itself calls die/croak/fail/etc.
@@ -237,7 +234,10 @@ sub run_check {
 
         $check->run;
 
-        return;
+        my @found = @{$check->found};
+        $check->found([]);
+
+        return @found;
     }
 
     my @args = ($proc->pkg_name,$proc->pkg_type,$proc->info,$proc,$group);
